<?php
/*
* tremulous.php
*
* This script connects to a Tremulous server and prints information
* about the players that are currently connected.  
*
* To use this script, copy it to the same directory as your page and
* add the following code to your page:
*
* <?php
* include_once("tremulous.php");
* tremulous_draw_players_table("your server name or IP", "your server port");
* ?>
*
* If don't provide server name or port parameters, "127.0.0.1" and
* "30720" will be used.
*
* Brian Rhodes bgr(at)linespeed.net
* Tony J. White tjw(at)tjw.org
* Ryan 'Dr. Zeebok' Allgaier ryan(at)rahs.net
*/

function tremulous_strip_colors($in) {
	$in = htmlspecialchars($in);

	$match = false;
	for($i = 0; $i < strlen($in)-1; $i++)
	{
		if(substr($in, $i, 1) == '^')
		{
			$next = substr($in, $i+1, 1);
			if($next != '^')
			{
				$num = (ord(substr($in, $i+1, 1))) % 8;
				switch($num)
				{
				case 0: $col = "black"; break;
				case 1: $col = "red"; break;
				case 2: $col = "green"; break;
				case 3: $col = "yellow"; break;
				case 4: $col = "blue"; break;
				case 5: $col = "cyan"; break;
				case 6: $col = "magenta"; break;
				case 7: $col = "white"; break;
				}
				$in = substr($in, 0, $i) . "</span><span style=\"color:$col;\">" . substr($in, $i+2, strlen($in));
			}
		}
	}
	$in = "<span style=\"color:white;\">" . $in . "</span>";

    return $in;
}

function tremulous_get_from_pk3($name)
{
	$CACHEDIR = "/home/mdoison/public_html/fsb2/tremcache";
	$MAPDIR = "/srv/tremulous/maps";

	$dh = opendir($MAPDIR);
	while($filename = readdir($dh))
	{
		$zh = zip_open("$MAPDIR/$filename");
		if(is_int($zh))
			continue;

		while($zip_entry = zip_read($zh))
		{
			if(strtolower(zip_entry_name($zip_entry)) == "levelshots/$name.jpg")
			{
				zip_entry_open($zh, $zip_entry, 'r');

				$size = zip_entry_filesize($zip_entry);
				$len = 0;
				$picture = "";
				while($len < $size)
				{
					$picture .= zip_entry_read($zip_entry, 1024);
					$len += 1024;
				}

				zip_entry_close($zip_entry);
				zip_close($zh);
				closedir($dh);

				$image = imagecreatefromstring($picture);

				$width = imagesx($image);
				$height = imagesy($image);
				$newwidth = 650;
				$newheight = 488;

				$newimage = imagecreatetruecolor($newwidth, $newheight);
				if(imagecopyresized($newimage, $image, 0, 0, 0, 0, $newwidth, $newheight, $width, $height))
					imagejpeg($newimage, "$CACHEDIR/$name.jpg");

				return true;
			}
		}

		zip_close($zh);
	}

	closedir($dh);

	return false;
}

function tremulous_precache_map($name)
{
	$CACHEDIR = "/home/mdoison/public_html/fsb2/tremcache";
	$CACHEURL = "tremcache";

	$name = strtolower($name);

	if(file_exists("$CACHEDIR/$name.jpg"))
	{
		return "$CACHEURL/$name.jpg";
	}
	else
	{
		tremulous_get_from_pk3($name);
	}

	return "$CACHEURL/$name.jpg";
}

function tremulous_get_players($server, $port)
{
    $fp = fsockopen("udp://".$server, $port, $errno, $errstr, 2);
    if(!$fp)
        return array();
    
    $status_str = "xxxxgetstatus";
    for($i=0;$i<4;$i++) $status_str[$i] = pack("v", 0xff);
    fwrite($fp, $status_str);
    socket_set_timeout($fp, 2);
    $data_full = fread($fp, 10000);
	if(!$data_full)
		return array();

    $data_full = substr($data_full, 19);
    $data = explode("\n", $data_full);
    fclose($fp);
    $server_data = explode("\\", $data[0]);

    for ($i=1; $i<count($server_data); $i+=2) {
        $server_status[$server_data[$i]] = $server_data[$i+1];
        if($server_data[$i] == "mapname") {
            $map = $server_data[$i+1];
        }
        elseif($server_data[$i] == "P") {
            $P = str_replace("-", "", $server_data[$i+1]);
        }
    }

    $i = 1;
    next($data); // skip settings
	$human = array();
	$alien = array();
	$spec = array();
    while(list(,$p) = each($data)) {
        if(preg_match("/^(-\d+|\d+) (\d+) \"(.*)\"$/", $p, $m)) {
            $pinfo = array(
                "kills" => $m[1],
                "ping" => $m[2],
                "raw_name" => $m[3],
                "name" => tremulous_strip_colors($m[3]),
                "colored_name" =>
                    tremulous_strip_colors($m[3]));
            if($P[$i-1] == 2) {
                $human[] = $pinfo;
            }
            elseif($P[$i-1] == 1) {
                $alien[] = $pinfo;
            }
            else {
                $spec[] = $pinfo;
            }
            $i++;
        }
    }

    return array(
        "alien_players" => $alien,
        "human_players" => $human,
        "spec_players" => $spec,
        "map" => $map
        );
}

function tremulous_draw_players_table($server, $port)
{
    $players = tremulous_get_players($server, $port);
	if(empty($players))
		return "<b>Serveur inactif</b>";
	$max = max(count($players["human_players"]), count($players["alien_players"]));
	$map = tremulous_precache_map($players["map"]);
	$ret = "
		<table align='center' style='width:650;height:488px;background-repeat: no-repeat;border:3px solid black;' border='0' width='650' bgcolor='#000000' background='" . $map . "'> <tr style='background:none'> <td valign='top' align='center'> <br/> <br/> <!--Aliens    Stages    Humans--> <table width='640' border='0' style='color:white;'> <tr style='background:none'> <td colspan='3' align='left'> <table width='100%' border='0' style='border:1px solid white;' cellspacing='0'> <tr style='background:none'> <td> <div style='color: #ffffff; font-size: 24px; font-weight: bold;text-align:left;'> &nbsp;Aliens &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <span style='font-size : 20px; font-weight : normal;'> </span> </div> </td> <td> <div style='color: #ffffff; font-size: 24px; font-weight: bold;text-align:right;'> <span style='font-size : 20px; font-weight : normal;'> </span> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; Humans&nbsp; </div> </td> </tr> </table> </td> </tr> <tr style='background:none'> <td> <!-- Head lines--> <table width='100%' border='0' style='border:1px solid white;font-size: 14px;color:#ffff00;' cellspacing='0'> <tr style='background:none'> <td align='left' width='240'>Player</td> <td align='right' width='40'>Kills</td> <td align='right' width='40'>Ping</td> </tr> </table> </td> <td rowspan='12'>&nbsp;</td> <td> <table width='100%' border='0' style='border:1px solid white;font-size: 14px;color:#ffff00;' cellspacing='0'> <tr style='background:none'> <td align='left' width='240'>Player</td> <td align='right' width='40'>Kills</td> <td align='right' width='40'>Ping</td> </tr> </table> </td> </tr> <tr style='background:none'> <td valign='top'> <!-- alien --> <table width='310' border='0' cellspacing='0' style='table-layout:fixed;border:1px solid white;'> " . tremulous_draw_team_players($players["alien_players"], $max) . " </table> </td> <td valign='top'> <!-- human --> <table width='310' border='0' cellspacing='0' style='table-layout:fixed;border:1px solid white;'> " . tremulous_draw_team_players($players["human_players"], $max) . " </table> </td> </tr> <tr style='background:none'> <td colspan='3'> <table align='center' border='0' width='100%' style='border:1px solid white;color:white;' cellspacing='0'> <tr style='background:none'> <td> <table width='100%' border='0' cellspacing='0' cellpadding='0'> <tr style='background:none'> <td style='font-size: 16px;font-weight:bold;color:#ffffff;'>Spectating:&nbsp;</td> <td style='font-size : 16px;' width='100%'> <marquee scrolldlay='1' scrollamount='3'> " . tremulous_draw_team_spectators($players["spec_players"]) . " </marquee> </td> </tr> </table> </td> </tr> </table> </td> </tr> </table> </td> </tr> </table> </body>";

	return $ret;
}

function tremulous_draw_team_players($players, $max)
{
	if($max < 12)
		$max = 12;

    $sort_type = SORT_NUMERIC;
    $sort_ord = SORT_DESC;
	if(is_array($players)) {
		$sort = array();
		while (list(,$v) = each($players))
			$sort[] = strtolower($v["kills"]);
		reset($players);
		array_multisort($sort, $sort_type, $sort_ord, $players);
	}

	$ret = "";

    if(is_array($players))
	{
		$count = 0;
		while(list(,$player) = each($players))
		{
			$ret .= " <tr style='background:none'> <td align='left' style='width: 76%; font-size: 14px;'> <div style='overflow: hidden;white-space: nowrap;'> " . $player["colored_name"] . " </div> </td> <td style='width: 12%;font-size: 14px;text-align:right;color:#ffffff;'> " . $player["kills"] . " </td> <td style='width: 12%;font-size: 14px;text-align:right;color:#ffffff;'> " . $player["ping"] . " </td> </tr>";
			$count++;
		}
		for($i = $count; $i < $max; $i++)
		{
			$ret .= " <tr style='background:none'><td colspan='3' style='font-size : 14px;'>&nbsp; </td></tr>";
		}
    }
	
	return $ret;
}

function tremulous_draw_team_spectators($players)
{
	$ret = "";

	if(is_array($players))
		while(list(,$player) = each($players))
		{
			$ret .= " <span style='color: #FFFFFF;'> " . $player["colored_name"] . " </span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
		}

	return $ret;
}

